from setuptools import setup
setup(name='hhqtrain',
      version='0.0.1',
      description='clinical trial information retriver',
      url='http://github.com/',
      author='hhq',
      author_email='779613975@qq.com',
      license='MIT',
      packages=['hhqtrain'],
      zip_safe=False)